@@ -28,6 +28,7 @@ uxbindir=${uxbindir:=/opt/puppetlabs/bin}
 # symlinks of server binaries
 symbindir=${symbindir:=/opt/puppetlabs/server/bin}
 app_prefix=${app_prefix:=/opt/puppetlabs/server/apps/${real_name}}
+dest_apps_dir="${DESTDIR}${app_prefix}"
 app_data=${app_data:=/opt/puppetlabs/server/data/${real_name}}
 app_logdir=${app_logdir:=/var/log/puppetlabs/${real_name}}
 system_config_dir=${system_config_dir:=${app_prefix}/config}
@@ -131,43 +132,47 @@ function task_install_source_deb_systemd {
 # Install the ezbake package software. This step is used during RPM &
 # Debian packaging during the 'install' phases.
 function task_install {
-    install -d -m 0755 "${DESTDIR}${app_prefix}"
+    install -d -m 0755 "${dest_apps_dir}"
     install -d -m 0770 "${DESTDIR}${app_data}"
-    install -m 0644 puppetdb.jar "${DESTDIR}${app_prefix}"
-    install -m 0755 ext/ezbake-functions.sh "${DESTDIR}${app_prefix}"
-    install -m 0644 ext/ezbake.manifest "${DESTDIR}${app_prefix}"
+    install -m 0644 puppetdb.jar "${dest_apps_dir}"
+    install -m 0755 ext/ezbake-functions.sh "${dest_apps_dir}"
+    install -m 0644 ext/ezbake.manifest "${dest_apps_dir}"
     install -d -m 0755 "${DESTDIR}${projconfdir}/conf.d"
 
+  
 
-    install -m 0644 ext/config/logback.xml "${DESTDIR}${projconfdir}/logback.xml"
     install -m 0644 ext/config/bootstrap.cfg "${DESTDIR}${projconfdir}/bootstrap.cfg"
+    install -m 0644 ext/config/conf.d/config.ini "${DESTDIR}${projconfdir}/conf.d/config.ini"
+    install -m 0644 ext/config/conf.d/database.ini "${DESTDIR}${projconfdir}/conf.d/database.ini"
     install -m 0644 ext/config/conf.d/jetty.ini "${DESTDIR}${projconfdir}/conf.d/jetty.ini"
     install -m 0644 ext/config/conf.d/repl.ini "${DESTDIR}${projconfdir}/conf.d/repl.ini"
-    install -m 0644 ext/config/conf.d/database.ini "${DESTDIR}${projconfdir}/conf.d/database.ini"
-    install -m 0644 ext/config/conf.d/config.ini "${DESTDIR}${projconfdir}/conf.d/config.ini"
+    install -m 0644 ext/config/logback.xml "${DESTDIR}${projconfdir}/logback.xml"
     install -m 0644 ext/config/request-logging.xml "${DESTDIR}${projconfdir}/request-logging.xml"
 
-    install -d -m 0755 "${DESTDIR}${app_prefix}/scripts"
-    install -m 0755 install.sh "${DESTDIR}${app_prefix}/scripts"
+    install -d -m 0755 "${dest_apps_dir}/scripts"
+    install -m 0755 install.sh "${dest_apps_dir}/scripts"
 
-    install -d -m 0755 "${DESTDIR}${app_prefix}/cli"
-    install -d -m 0755 "${DESTDIR}${app_prefix}/cli/apps"
+    install -d -m 0755 "${dest_apps_dir}/cli"
+    install -d -m 0755 "${dest_apps_dir}/cli/apps"
     install -d -m 0755 "${DESTDIR}${bindir}"
     install -m 0755 "ext/bin/${real_name}" "${DESTDIR}${bindir}/${real_name}"
     install -d -m 0755 "${DESTDIR}${symbindir}"
     ln -s "../apps/${real_name}/bin/${real_name}" "${DESTDIR}${symbindir}/${real_name}"
     install -d -m 0755 "${DESTDIR}${uxbindir}"
     ln -s "../server/apps/${real_name}/bin/${real_name}" "${DESTDIR}${uxbindir}/${real_name}"
-    install -m 0755 ext/cli/reload "${DESTDIR}${app_prefix}/cli/apps/reload"
-    install -m 0755 ext/cli/foreground "${DESTDIR}${app_prefix}/cli/apps/foreground"
-    install -m 0755 ext/cli/start "${DESTDIR}${app_prefix}/cli/apps/start"
-    install -m 0755 ext/cli/ssl-setup "${DESTDIR}${app_prefix}/cli/apps/ssl-setup"
-    install -m 0755 ext/cli/export "${DESTDIR}${app_prefix}/cli/apps/export"
-    install -m 0755 ext/cli/config-migration "${DESTDIR}${app_prefix}/cli/apps/config-migration"
-    install -m 0755 ext/cli/foreground "${DESTDIR}${app_prefix}/cli/apps/foreground"
-    install -m 0755 ext/cli/stop "${DESTDIR}${app_prefix}/cli/apps/stop"
-    install -m 0755 ext/cli/anonymize "${DESTDIR}${app_prefix}/cli/apps/anonymize"
-    install -m 0755 ext/cli/import "${DESTDIR}${app_prefix}/cli/apps/import"
+    install -m 0755 ext/cli/foreground "${dest_apps_dir}/cli/apps/foreground"
+    install -m 0755 ext/cli/reload "${dest_apps_dir}/cli/apps/reload"
+    install -m 0755 ext/cli/stop "${dest_apps_dir}/cli/apps/stop"
+    install -m 0755 ext/cli/start "${dest_apps_dir}/cli/apps/start"
+    install -m 0755 ext/cli/anonymize "${dest_apps_dir}/cli/apps/anonymize"
+    install -m 0755 ext/cli/config-migration "${dest_apps_dir}/cli/apps/config-migration"
+    install -m 0755 ext/cli/foreground "${dest_apps_dir}/cli/apps/foreground"
+    install -m 0755 ext/cli/ssl-setup "${dest_apps_dir}/cli/apps/ssl-setup"
+
+if [ -e "ext/cli_defaults/cli-defaults.sh" ]; then
+    install -m 0755 ext/cli_defaults/cli-defaults.sh "${dest_apps_dir}/cli/"
+fi
+
     install -d -m 0755 "${DESTDIR}${rundir}"
     install -d -m 700 "${DESTDIR}${app_logdir}"
 }
@@ -189,22 +194,22 @@ function task_termini {
     # This also allows us to prefer PE ruby over system ruby, so this will work
     # with PE as well
     local rubylibdir=${rubylibdir:=$(/opt/puppetlabs/puppet/bin/ruby -rrbconfig -e "puts RbConfig::CONFIG['vendordir']")}
-    install -Dm 0644 puppet/indirector/node/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/indirector/node/puppetdb.rb"
-    install -Dm 0644 puppet/indirector/facts/puppetdb_apply.rb "${DESTDIR}${rubylibdir}/puppet/indirector/facts/puppetdb_apply.rb"
-    install -Dm 0644 puppet/indirector/facts/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/indirector/facts/puppetdb.rb"
+    install -Dm 0644 puppet/face/node/deactivate.rb "${DESTDIR}${rubylibdir}/puppet/face/node/deactivate.rb"
+    install -Dm 0644 puppet/face/node/status.rb "${DESTDIR}${rubylibdir}/puppet/face/node/status.rb"
+    install -Dm 0644 puppet/functions/puppetdb_query.rb "${DESTDIR}${rubylibdir}/puppet/functions/puppetdb_query.rb"
     install -Dm 0644 puppet/indirector/catalog/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/indirector/catalog/puppetdb.rb"
+    install -Dm 0644 puppet/indirector/facts/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/indirector/facts/puppetdb.rb"
+    install -Dm 0644 puppet/indirector/facts/puppetdb_apply.rb "${DESTDIR}${rubylibdir}/puppet/indirector/facts/puppetdb_apply.rb"
+    install -Dm 0644 puppet/indirector/node/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/indirector/node/puppetdb.rb"
     install -Dm 0644 puppet/indirector/resource/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/indirector/resource/puppetdb.rb"
+    install -Dm 0644 puppet/reports/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/reports/puppetdb.rb"
+    install -Dm 0644 puppet/util/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb.rb"
+    install -Dm 0644 puppet/util/puppetdb/atom.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/atom.rb"
+    install -Dm 0644 puppet/util/puppetdb/char_encoding.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/char_encoding.rb"
     install -Dm 0644 puppet/util/puppetdb/command.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/command.rb"
     install -Dm 0644 puppet/util/puppetdb/command_names.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/command_names.rb"
     install -Dm 0644 puppet/util/puppetdb/config.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/config.rb"
     install -Dm 0644 puppet/util/puppetdb/http.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/http.rb"
-    install -Dm 0644 puppet/util/puppetdb/char_encoding.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/char_encoding.rb"
-    install -Dm 0644 puppet/util/puppetdb/atom.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb/atom.rb"
-    install -Dm 0644 puppet/util/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/util/puppetdb.rb"
-    install -Dm 0644 puppet/reports/puppetdb.rb "${DESTDIR}${rubylibdir}/puppet/reports/puppetdb.rb"
-    install -Dm 0644 puppet/face/node/deactivate.rb "${DESTDIR}${rubylibdir}/puppet/face/node/deactivate.rb"
-    install -Dm 0644 puppet/face/node/status.rb "${DESTDIR}${rubylibdir}/puppet/face/node/status.rb"
-    install -Dm 0644 puppet/functions/puppetdb_query.rb "${DESTDIR}${rubylibdir}/puppet/functions/puppetdb_query.rb"
 }
 
 function task_defaults_redhat {
