@@ -1,6 +1,6 @@
-(def pdb-version "4.4.0")
-(def puppetserver-version "2.7.2")
-(def clj-parent-version "0.6.0")
+(def pdb-version "5.0.1")
+(def puppetserver-version "5.0.0")
+(def clj-parent-version "1.2.1")
 
 (defn deploy-info
   "Generate deployment information from the URL supplied and the username and
@@ -13,10 +13,8 @@
 
 (def i18n-version "0.8.0")
 
-(def pdb-jvm-opts
-  (case (System/getProperty "java.specification.version")
-    "1.7" ["-XX:MaxPermSize=200M"]
-    []))
+(def need-permgen?
+  (= "1.7" (System/getProperty "java.specification.version")))
 
 (defproject puppetlabs/puppetdb pdb-version
   :description "Puppet-integrated catalog and fact storage"
@@ -61,7 +59,7 @@
                  [puppetlabs/trapperkeeper-status]
 
                  ;; Various
-                 [cheshire]
+                 [cheshire "5.7.1"]
                  [clj-stacktrace]
                  [clj-time]
                  [com.rpl/specter "0.5.7"]
@@ -102,7 +100,9 @@
                  [org.apache.commons/commons-compress "1.10"]
                  [ring/ring-core :exclusions [javax.servlet/servlet-api org.clojure/tools.reader]]]
 
-  :jvm-opts ~pdb-jvm-opts
+  :jvm-opts ~(if need-permgen?
+              ["-XX:MaxPermSize=200M"]
+              [])
 
   :repositories [["releases" "http://nexus.delivery.puppetlabs.net/content/repositories/releases/"]
                  ["snapshots" "http://nexus.delivery.puppetlabs.net/content/repositories/snapshots/"]]
@@ -151,8 +151,7 @@
                    :injections [(do
                                   (require 'schema.core)
                                   (schema.core/set-fn-validation! true))]}
-             :ezbake {:dependencies ^:replace [;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-                                               ;; NOTE: we need to explicitly pass in `nil` values
+             :ezbake {:dependencies ^:replace [;; NOTE: we need to explicitly pass in `nil` values
                                                ;; for the version numbers here in order to correctly
                                                ;; inherit the versions from our parent project.
                                                ;; This is because of a bug in lein 2.7.1 that
@@ -162,20 +161,26 @@
                                                ;; https://github.com/technomancy/leiningen/issues/2216
                                                ;; Hopefully we can remove those `nil`s (if we care)
                                                ;; and this comment when lein 2.7.2 is available.
-                                               ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
                                                ;; we need to explicitly pull in our parent project's
                                                ;; clojure version here, because without it, lein
                                                ;; brings in its own version, and older versions of
                                                ;; lein depend on clojure 1.6.
                                                [org.clojure/clojure nil]
+
+                                               ;; This circular dependency is required because of a bug in
+                                               ;; ezbake (EZ-35); without it, bootstrap.cfg will not be included
+                                               ;; in the final package.
                                                [puppetlabs/puppetdb ~pdb-version]
-                                               [puppetlabs/trapperkeeper-webserver-jetty9 nil]
                                                [org.clojure/tools.nrepl nil]]
                       :name "puppetdb"
-                      :plugins [[puppetlabs/lein-ezbake "1.1.6"]]}
+                      :plugins [[puppetlabs/lein-ezbake "1.4.0"]]}
              :testutils {:source-paths ^:replace ["test"]}
-             :ci {:plugins [[lein-pprint "1.1.1"]]}}
+             :ci {:plugins [[lein-pprint "1.1.1"]]}
+             :test {:jvm-opts ~(if need-permgen?
+                                 ;; integration tests cycle jruby a lot, which chews through permgen
+                                 ^:replace ["-XX:MaxPermSize=500M"]
+                                 [])}}
 
   :jar-exclusions [#"leiningen/"]
 
