@@ -76,7 +76,8 @@
             [slingshot.slingshot :refer [throw+ try+]]
             [clojure.core.async :as async]
             [puppetlabs.puppetdb.command :as cmd]
-            [puppetlabs.puppetdb.queue :as queue])
+            [puppetlabs.puppetdb.queue :as queue]
+            [puppetlabs.i18n.core :refer [trs tru]])
   (:import [javax.jms ExceptionListener]
            [java.util.concurrent.locks ReentrantLock]
            [org.joda.time Period]))
@@ -84,10 +85,39 @@
 (def cli-description "Main PuppetDB daemon")
 (def database-metrics-registry (get-in metrics/metrics-registries [:database :registry]))
 
-;; ## Wiring
-;;
-;; The following functions setup interaction between the main
-;; PuppetDB components.
+(def clean-options
+  #{"expire_nodes" "purge_nodes" "purge_reports" "package_gc" "other"})
+
+(def purge-nodes-opts-schema {:batch_limit s/Int})
+
+(def clean-request-schema
+  ;; i.e. a possibly empty collection of elements, each of which must
+  ;; either be a string from clean-options or a purge_nodes command
+  ;; with an options map.
+  [(s/if string?
+     (apply s/enum clean-options)
+     [(s/one (s/eq "purge_nodes") "command")
+      (s/one purge-nodes-opts-schema "options")])])
+
+(defn reduce-clean-request [request]
+  "Converts the incoming vector of requests to a map of requests to
+  their options, where the last one of each kind wins.
+  e.g. [\"purge_nodes\" \"purge_reports\" {\"purge_nodes\" {...}}]
+  becomes #{{\"purge_reports\" true} {\"purge_nodes\" {...}}}."
+  (into {} (map (fn [x]
+                  (if (string? x)
+                    [x true]
+                    x))
+            request)))
+
+(defn reduced-clean-request->status [request]
+  (->> request keys sort
+       (map {"expire_nodes" "expiring-nodes"
+             "purge_nodes" "purging-nodes"
+             "purge_reports" "purging-reports"
+             "gc_packages" "package-gc"
+             "other" "other"})
+       (str/join " ")))
 
 (defn auto-expire-nodes!
   "Expire nodes which haven't had any activity (catalog/fact submission)
@@ -101,24 +131,28 @@
              (format-period node-ttl))
      (jdbc/with-transacted-connection db
        (doseq [node (scf-store/expire-stale-nodes node-ttl)]
-         (log/infof "Auto-expired node %s" node))))
+         (log/info (trs "Auto-expired node {0}" node)))))
     (catch Exception e
-      (log/error e "Error while deactivating stale nodes"))))
+      (log/error e (trs "Error while deactivating stale nodes")))))
 
-(defn purge-nodes!
-  "Delete nodes which have been *deactivated or expired* longer than
-  `node-purge-ttl`."
-  [node-purge-ttl db]
-  {:pre [(map? db)
-         (period? node-purge-ttl)]}
+(defn-validated purge-nodes!
+  "Deletes nodes which have been *deactivated or expired* longer than
+  node-purge-ttl.  Deletes at most batch_limit nodes if opts is a map,
+  all relevant nodes otherwise."
+  [node-purge-ttl :- (s/pred period?)
+   opts :- (s/if map? purge-nodes-opts-schema (s/eq true))
+   db :- (s/pred map?)]
   (try
     (kitchensink/demarcate
      (format "purge deactivated and expired nodes (threshold: %s)"
              (format-period node-purge-ttl))
-     (jdbc/with-transacted-connection db
-       (scf-store/purge-deactivated-and-expired-nodes! (ago node-purge-ttl))))
+     (let [horizon (ago node-purge-ttl)]
+       (jdbc/with-transacted-connection db
+         (if-let [limit (and (map? opts) (:batch_limit opts))]
+           (scf-store/purge-deactivated-and-expired-nodes! horizon limit)
+           (scf-store/purge-deactivated-and-expired-nodes! horizon)))))
     (catch Exception e
-      (log/error e "Error while purging deactivated and expired nodes"))))
+      (log/error e (trs "Error while purging deactivated and expired nodes")))))
 
 (defn sweep-reports!
   "Delete reports which are older than than `report-ttl`."
@@ -132,7 +166,16 @@
      (jdbc/with-transacted-connection db
        (scf-store/delete-reports-older-than! (ago report-ttl))))
     (catch Exception e
-      (log/error e "Error while sweeping reports"))))
+      (log/error e (trs "Error while sweeping reports")))))
+
+(defn gc-packages! [ttl db]
+  {:pre [(map? db)]}
+  (try
+    (kitchensink/demarcate (trs "package gc")
+      (jdbc/with-transacted-connection db
+        (scf-store/delete-unassociated-packages!)))
+    (catch Exception e
+      (log/error e (trs "Error while running package gc")))))
 
 (defn garbage-collect!
   "Perform garbage collection on `db`, which means deleting any orphaned data.
@@ -142,19 +185,10 @@
   {:pre [(map? db)]}
   (try
     (kitchensink/demarcate
-      "database garbage collection"
+      (trs "database garbage collection")
       (scf-store/garbage-collect! db))
     (catch Exception e
-      (log/error e "Error during garbage collection"))))
-
-(def clean-options #{"expire_nodes" "purge_nodes" "purge_reports" "other"})
-
-(defn clean-options->status [options]
-  (str/join " " (map {"expire_nodes" "expiring-nodes"
-                      "purge_nodes" "purging-nodes"
-                      "purge_reports" "purging-reports"
-                      "other" "other"}
-                     (sort options))))
+      (log/error e (trs "Error during garbage collection")))))
 
 (def admin-metrics-registry
   (get-in metrics/metrics-registries [:admin :registry]))
@@ -169,16 +203,15 @@
    :node-expirations (counter admin-metrics-registry "node-expirations")
    :node-purges (counter admin-metrics-registry "node-purges")
    :report-purges (counter admin-metrics-registry "report-purges")
+   :package-gcs (counter admin-metrics-registry "package-gcs")
    :other-cleans (counter admin-metrics-registry "other-cleans")
 
    :node-expiration-time (timer admin-metrics-registry ["node-expiration-time"])
    :node-purge-time (timer admin-metrics-registry ["node-purge-time"])
    :report-purge-time (timer admin-metrics-registry ["report-purge-time"])
+   :package-gc-time (timer admin-metrics-registry "package-gc-time")
    :other-clean-time (timer admin-metrics-registry ["other-clean-time"])})
 
-(def clean-request-schema
-  [(apply s/enum clean-options)])
-
 (defn- clear-clean-status!
   "Clears the clean status (as a separate function to support tests)."
   []
@@ -196,23 +229,29 @@
    ;; Later, the values might be maps, i.e. {:limit 1000}
    request :- clean-request-schema]
   (when-not (.isHeldByCurrentThread lock)
-    (throw (IllegalStateException. "cleanup lock is not already held")))
-  (let [request (if (empty? request) clean-options (set request))
-        status (clean-options->status request)]
+    (throw (IllegalStateException. (tru "cleanup lock is not already held"))))
+  (let [request (reduce-clean-request (if (empty? request)
+                                        clean-options  ; clean everything
+                                        request))
+        status (reduced-clean-request->status request)]
     (try
       (reset! clean-status status)
       (when (request "expire_nodes")
         (time! (:node-expiration-time admin-metrics)
                (auto-expire-nodes! node-ttl db))
         (counters/inc! (:node-expirations admin-metrics)))
-      (when (request "purge_nodes")
+      (when-let [opts (request "purge_nodes")]
         (time! (:node-purge-time admin-metrics)
-               (purge-nodes! node-purge-ttl db))
+               (purge-nodes! node-purge-ttl opts db))
         (counters/inc! (:node-purges admin-metrics)))
       (when (request "purge_reports")
         (time! (:report-purge-time admin-metrics)
                (sweep-reports! report-ttl db))
         (counters/inc! (:report-purges admin-metrics)))
+      (when (request "gc_packages")
+        (time! (:package-gc-time admin-metrics)
+               (gc-packages! db))
+        (counters/inc! (:package-gcs admin-metrics)))
       ;; It's important that this go last to ensure anything referencing
       ;; an env or resource param is purged first.
       (when (request "other")
@@ -246,14 +285,14 @@
                                           (version/check-for-updates! read-db))
              job-pool
              :desc "A reoccuring job to checkin the PuppetDB version"))
-    (log/debug "Skipping update check on Puppet Enterprise")))
+    (log/debug (trs "Skipping update check on Puppet Enterprise"))))
 
 (defn stop-puppetdb
   "Shuts down PuppetDB, releasing resources when possible.  If this is
   not a normal shutdown, emergency? must be set, which currently just
   produces a fatal level level log message, instead of info."
   [context]
-  (log/info "Shutdown request received; puppetdb exiting.")
+  (log/info (trs "Shutdown request received; puppetdb exiting."))
   (when-let [ds (get-in context [:shared-globals :scf-write-db :datasource])]
     (.close ds))
   (when-let [ds (get-in context [:shared-globals :scf-read-db :datasource])]
@@ -265,17 +304,23 @@
   context)
 
 (defn initialize-schema
-  "Ensure the database is migrated to the latest version, and warn if
-  it's deprecated, log and exit if it's unsupported.
-
-  This function will return true iff any migrations were run."
+  "Ensures the database is migrated to the latest version, and returns
+  true iff any migrations were run.  Throws
+  {:type ::unsupported-database :current version :oldest version} if
+  the current database is not supported."
   [db-conn-pool config]
   (jdbc/with-db-connection db-conn-pool
-    (scf-store/validate-database-version #(System/exit 1))
-    @sutils/db-metadata
-    (let [migrated? (migrate! db-conn-pool)]
-      (indexes! config)
-      migrated?)))
+    (let [current (:version @sutils/db-metadata)
+          oldest (get-in config [:database :min-required-version]
+                           scf-store/oldest-supported-db)]
+      (when (neg? (compare current oldest))
+        (throw+ {:type ::unsupported-database
+                 :current current
+                 :oldest oldest}))
+      @sutils/db-metadata
+      (let [migrated? (migrate! db-conn-pool)]
+        (indexes! config)
+        migrated?))))
 
 (defn init-with-db
   "All initialization operations needing a database connection should
@@ -283,9 +328,10 @@
   `write-db-config` that will hang until it is able to make a
   connection to the database. This covers the case of the database not
   being fully started when PuppetDB starts. This connection pool will
-  be opened and closed within the body of this function.
-
-  This function will return true iff any migrations were run."
+  be opened and closed within the body of this function.  Returns true
+  iff any migrations were run.  Throws
+  {:type ::unsupported-database :current version :oldest version} if the
+  current database is not supported."
   [write-db-config config]
   (loop [db-spec (assoc write-db-config
                         ;; Block waiting to grab a connection
@@ -299,11 +345,39 @@
                    (initialize-schema db-pool-map config)
                    ::success))
                (catch java.sql.SQLTransientConnectionException e
-                 (log/error e "Error while attempting to create connection pool")))]
+                 (log/error e (trs "Error while attempting to create connection pool"))))]
       result
       (recur db-spec))))
 
+(defn db-config->clean-request
+  [config]
+  (let [seconds-pos? (comp pos? to-seconds)]
+    (filter identity
+            [(when (some-> (:node-ttl config) seconds-pos?)
+               "expire_nodes")
+             (when (some-> (:node-purge-ttl config) seconds-pos?)
+               (let [limit (:node-purge-gc-batch-limit config)]
+                 (if (zero? limit)
+                            "purge_nodes"
+                            ["purge_nodes" {:batch_limit limit}])))
+             (when (some-> (:report-ttl config) seconds-pos?)
+               "purge_reports")
+             "other"])))
+
+(defn collect-garbage
+  [db clean-lock config clean-request]
+  (.lock clean-lock)
+  (try
+    (try
+      (clean-up db clean-lock config clean-request)
+      (catch Exception ex
+        (log/error ex)))
+    (finally
+      (.unlock clean-lock))))
+
 (defn start-puppetdb
+  "Throws {:type ::unsupported-database :current version :oldest version} if
+  the current database is not supported."
   [context config service get-registered-endpoints]
   {:pre [(map? context)
          (map? config)]
@@ -321,7 +395,7 @@
                                         database-metrics-registry)]
 
     (when-let [v (version/version)]
-      (log/infof "PuppetDB version %s" v))
+      (log/info (trs "PuppetDB version {0}" v)))
 
     (init-with-db database config)
 
@@ -330,7 +404,10 @@
 
     ;; Error handling here?
     (let [stockdir (conf/stockpile-dir config)
-          command-chan (async/chan max-enqueued)
+          command-chan (async/chan
+                         (queue/sorted-command-buffer
+                          max-enqueued
+                          #(cmd/update-counter! :invalidated %1 %2 inc!)))
           [q load-messages] (queue/create-or-open-stockpile (conf/stockpile-dir config))
           globals {:scf-read-db read-db
                    :scf-write-db write-db
@@ -351,28 +428,9 @@
         (when-not disable-update-checking
           (maybe-check-for-updates config read-db job-pool))
         (when (pos? gc-interval-millis)
-          (let [seconds-pos? (comp pos? to-seconds)
-                what (filter identity
-                             [(when-let [node-ttl (:node-ttl database)]
-                                (when (seconds-pos? node-ttl)
-                                  "expire_nodes"))
-                              (when-let [node-purge-ttl (:node-purge-ttl database)]
-                                (when (seconds-pos? node-purge-ttl)
-                                  "purge_nodes"))
-                              (when-let [report-ttl (:report-ttl database)]
-                                (when (seconds-pos? report-ttl)
-                                  "purge_reports"))
-                              "other"])]
+          (let [request (db-config->clean-request database)]
             (interspaced gc-interval-millis
-                         (fn []
-                           (.lock clean-lock)
-                           (try
-                             (try
-                               (clean-up write-db clean-lock database what)
-                               (catch Exception ex
-                                 (log/error ex)))
-                             (finally
-                               (.unlock clean-lock))))
+                         #(collect-garbage write-db clean-lock database request)
                          job-pool)))
         (assoc context
                :job-pool job-pool
@@ -380,6 +438,15 @@
                :clean-lock clean-lock
                :command-loader command-loader)))))
 
+(defn db-unsupported-msg
+  "Returns a message describing which databases are supported."
+  [current oldest]
+  (trs "PostgreSQL {0}.{1} is no longer supported.  Please upgrade to {2}.{3}."
+       (first current)
+       (second current)
+       (first oldest)
+       (second oldest)))
+
 (defprotocol PuppetDBServer
   (shared-globals [this])
   (set-url-prefix [this url-prefix])
@@ -402,14 +469,25 @@
   that trapperkeeper will call on exit."
   PuppetDBServer
   [[:DefaultedConfig get-config]
-   [:WebroutingService add-ring-handler get-registered-endpoints]]
+   [:WebroutingService add-ring-handler get-registered-endpoints]
+   [:ShutdownService shutdown-on-error]]
   (init [this context]
 
         (doseq [{:keys [reporter]} (vals metrics/metrics-registries)]
           (jmx-reporter/start reporter))
         (assoc context :url-prefix (atom nil)))
   (start [this context]
-         (start-puppetdb context (get-config) this get-registered-endpoints))
+         (try+
+          (start-puppetdb context (get-config) this get-registered-endpoints)
+          (catch [:type ::unsupported-database] {:keys [current oldest]}
+            (let [msg (db-unsupported-msg current oldest)
+                  attn (utils/attention-msg msg)]
+              (utils/println-err attn)
+              (log/error attn)
+              ;; Until TK-445 is resolved, we won't be able to avoid the
+              ;; backtrace on exit this causes.
+              (shutdown-on-error (service-id this)
+                                 #(throw (Exception. msg)))))))
 
   (stop [this context]
         (doseq [{:keys [reporter]} (vals metrics/metrics-registries)]
