@@ -6,7 +6,7 @@
    and the format expected by the rest of the application."
   (:import [java.security KeyStore]
            [org.joda.time Minutes Days Period])
-  (:require [puppetlabs.i18n.core :as i18n]
+  (:require [puppetlabs.i18n.core :refer [trs]]
             [clojure.tools.logging :as log]
             [puppetlabs.kitchensink.core :as kitchensink]
             [puppetlabs.trapperkeeper.bootstrap :as tk-bootstrap]
@@ -34,7 +34,8 @@
 (defn warn-unknown-keys
   [schema data]
   (doseq [k (pls/unknown-keys schema data)]
-    (log/warnf "The configuration item `%s` does not exist and should be removed from the config." k)))
+    (log/warn
+     (trs "The configuration item `{0}` does not exist and should be removed from the config." k))))
 
 (defn warn-and-validate
   "Warns a user about unknown configurations items, removes them and validates the config."
@@ -77,10 +78,10 @@
   (merge database-config-in
          (all-optional
            {:gc-interval (pls/defaulted-maybe s/Int 60)
-            :dlo-compression-interval s/Int
             :report-ttl (pls/defaulted-maybe String "14d")
-            :node-purge-ttl (pls/defaulted-maybe String "0s")
-            :node-ttl (pls/defaulted-maybe String "0s")})))
+            :node-purge-ttl (pls/defaulted-maybe String "14d")
+            :node-purge-gc-batch-limit (pls/defaulted-maybe s/Int 25)
+            :node-ttl (pls/defaulted-maybe String "7d")})))
 
 (def database-config-out
   "Schema for parsed/processed database config"
@@ -109,9 +110,9 @@
   "Schema for parsed/processed database config that includes write database params"
   (merge database-config-out
          {:gc-interval Minutes
-          (s/optional-key :dlo-compression-interval) Minutes
           :report-ttl Period
           :node-purge-ttl Period
+          :node-purge-gc-batch-limit (s/constrained s/Int (complement neg?))
           :node-ttl Period}))
 
 (defn half-the-cores*
@@ -142,22 +143,26 @@
 (def command-processing-in
   "Schema for incoming command processing config (user defined) - currently incomplete"
   (all-optional
-    {:dlo-compression-threshold (pls/defaulted-maybe String "1d")
-     :threads (pls/defaulted-maybe s/Int half-the-cores)
-     :store-usage s/Int
-     :max-frame-size (pls/defaulted-maybe s/Int 209715200)
-     :temp-usage s/Int
-     :memory-usage s/Int
+    {:threads (pls/defaulted-maybe s/Int half-the-cores)
      :max-command-size (pls/defaulted-maybe s/Int (default-max-command-size))
-     :reject-large-commands (pls/defaulted-maybe String "false")}))
+     :reject-large-commands (pls/defaulted-maybe String "false")
+     :concurrent-writes (pls/defaulted-maybe s/Int (min half-the-cores 4))
+
+     ;; Deprecated
+     :max-frame-size (pls/defaulted-maybe s/Int 209715200)
+     :store-usage s/Int
+     :temp-usage s/Int
+     :memory-usage s/Int}))
 
 (def command-processing-out
   "Schema for parsed/processed command processing config - currently incomplete"
-  {:dlo-compression-threshold Period
-   :threads s/Int
-   :max-frame-size s/Int
+  {:threads s/Int
    :max-command-size s/Int
    :reject-large-commands Boolean
+   :concurrent-writes s/Int
+
+   ;; Deprecated
+   :max-frame-size s/Int
    (s/optional-key :memory-usage) s/Int
    (s/optional-key :store-usage) s/Int
    (s/optional-key :temp-usage) s/Int})
@@ -177,10 +182,12 @@
 
 (def developer-config-in
   (all-optional
-    {:pretty-print (pls/defaulted-maybe String "false")}))
+   {:pretty-print (pls/defaulted-maybe String "false")
+    :max-enqueued (pls/defaulted-maybe s/Int 1000000)}))
 
 (def developer-config-out
-  {:pretty-print Boolean})
+  {:pretty-print Boolean
+   :max-enqueued s/Int})
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Database config
@@ -193,8 +200,8 @@
   [{db-config :database :or {db-config {}} :as config}]
   (when (str/blank? (:subname db-config))
     (throw+
-     {:type ::cli-error
-      :message
+     {:kind ::cli-error
+      :msg
       (str "PuppetDB requires PostgreSQL."
            "  The [database] section must contain an appropriate"
            " \"//host:port/database\" subname setting.")}))
@@ -236,8 +243,22 @@
   [{:keys [developer] :as config :or {developer {}}}]
   (configure-section config :developer developer-config-in developer-config-out))
 
+(def retired-cmd-proc-keys
+  [:store-usage :max-frame-size :temp-usage :memory-usage])
+
+(defn warn-command-processing-retirements
+  [config]
+  (doseq [cmd-proc-key retired-cmd-proc-keys]
+    (when (get-in config [:command-processing cmd-proc-key])
+      (utils/println-err
+       (str (trs "The configuration item `{0}` in the [command-processing] section is retired, please remove this item from your config."
+                 (name cmd-proc-key))
+            " "
+            (trs "Consult the documentation for more details."))))))
+
 (defn configure-command-processing
   [config]
+  (warn-command-processing-retirements config)
   (configure-section config :command-processing command-processing-in command-processing-out))
 
 (defn convert-config
@@ -246,7 +267,6 @@
   [config]
   (-> config
       (configure-section :database write-database-config-in write-database-config-out)
-      (update :database #(utils/assoc-when % :dlo-compression-interval (:gc-interval %)))
       configure-read-db
       configure-command-processing
       configure-puppetdb))
@@ -265,23 +285,27 @@
     (cond
      (nil? vardir)
      (throw (IllegalArgumentException.
-             "Required setting 'vardir' is not specified. Please set it to a writable directory."))
+             (format "%s %s"
+                     (trs "Required setting ''vardir'' is not specified.")
+                     (trs "Please set it to a writable directory."))))
 
      (not (.isAbsolute vardir))
      (throw (IllegalArgumentException.
-             (format "Vardir %s must be an absolute path." vardir)))
+             (trs "Vardir {0} must be an absolute path." vardir)))
 
      (not (.exists vardir))
      (throw (java.io.FileNotFoundException.
-             (format "Vardir %s does not exist. Please create it and ensure it is writable." vardir)))
+             (format "%s %s"
+                     (trs "Vardir {0} does not exist." vardir)
+                     (trs "Please create it and ensure it is writable."))))
 
      (not (.isDirectory vardir))
      (throw (java.io.FileNotFoundException.
-             (format "Vardir %s is not a directory." vardir)))
+             (trs "Vardir {0} is not a directory." vardir)))
 
      (not (.canWrite vardir))
      (throw (java.io.FileNotFoundException.
-             (format "Vardir %s is not writable." vardir)))
+             (trs "Vardir {0} is not writable." vardir)))
 
      :else
      config)))
@@ -295,7 +319,7 @@
   (let [lower-product-name (str/lower-case product-name)]
     (when-not (#{"puppetdb" "pe-puppetdb"} lower-product-name)
       (throw (IllegalArgumentException.
-              (format "product-name %s is illegal; either puppetdb or pe-puppetdb are allowed" product-name))))
+              (trs "product-name {0} is illegal; either puppetdb or pe-puppetdb are allowed" product-name))))
     lower-product-name))
 
 (defn configure-globals
@@ -314,24 +338,25 @@
   (doseq [param [:classname :subprotocol]]
     (when (get-in config-data [:database param])
       (utils/println-err
-       (format "The [database] %s setting has been retired and will be ignored."
-               (name param)))))
+       (trs "The [database] {0} setting has been retired and will be ignored."
+            (name param)))))
   (when (get-in config-data [:global :catalog-hash-conflict-debugging])
-    (utils/println-err (str "The configuration item `catalog-hash-conflict-debugging`"
-                            " in the [global] section is retired,"
-                            " please remove this item from your config."
-                            " Consult the documentation for more details.")))
+    (utils/println-err (format "%s %s"
+                               (trs "The configuration item `catalog-hash-conflict-debugging` in the [global] section is retired, lease remove this item from your config.")
+                               (trs"Consult the documentation for more details."))))
+
   (when (get-in config-data [:repl])
-    (utils/println-err (str "The configuration block [repl] is now retired and will be ignored."
-                            " Use [nrepl] instead. Consult the documentation for more details.")))
+    (utils/println-err (format "%s %s %s"
+                               (trs "The configuration block [repl] is now retired and will be ignored.")
+                               (trs "Use [nrepl] instead.")
+                               (trs "Consult the documentation for more details."))))
+
   (when (get-in config-data [:global :url-prefix])
-    (utils/println-err (str "The configuration item `url-prefix` in the [global] section is retired,"
-                            " please remove this item from your config."
-                            " PuppetDB has a non-configurable context route of `/pdb`."
-                            " Consult the documentation for more details."))
-    (flush)
-    (binding [*out* *err*] (flush))
-    (System/exit 1)) ; cf. PDB-2053
+    (utils/println-err (format "%s %s %s"
+                               (trs "The configuration item `url-prefix` in the [global] section is retired, please remove this item from your config.")
+                               (trs "PuppetDB has a non-configurable context route of `/pdb`.")
+                               (trs "Consult the documentation for more details.")))
+    (utils/flush-and-exit 1)) ; cf. PDB-2053
   config-data)
 
 (def default-web-router-config
@@ -365,8 +390,8 @@
     (doseq [[svc route] filtered-web-router-config]
       (when (get-in config-data [:web-router-service svc])
         (utils/println-err
-         (i18n/trs "Configuring the route for `{0}` is not allowed. The default route is `{1}` and server is `{2}`."
-                   svc (:route route) (:server route)))))
+         (trs "Configuring the route for `{0}` is not allowed. The default route is `{1}` and server is `{2}`."
+              svc (:route route) (:server route)))))
     ;; We override the users settings as to make the above routes *not*
     ;; configurable
     (-> config-data
@@ -392,7 +417,7 @@
 (defn hook-tk-parse-config-data
   "This is a robert.hooke compatible hook that is designed to intercept
    trapperkeeper configuration before it is used, so that we may munge &
-   customize it.  It may throw {:type ::cli-error :message m}."
+   customize it.  It may throw {:kind ::cli-error :msg m}."
   [f args]
   (adjust-and-validate-tk-config (f args)))
 
@@ -439,11 +464,8 @@
   [config]
   (get-in config [:command-processing :max-command-size]))
 
-(defn mq-dir [config]
-  (str (io/file (get-in config [:global :vardir]) "mq")))
-
-(defn mq-discard-dir [config]
-  (str (io/file (mq-dir config) "discard")))
+(defn stockpile-dir [config]
+  (str (io/file (get-in config [:global :vardir]) "stockpile")))
 
 (defprotocol DefaultedConfig
   (get-config [this]))
