@@ -439,11 +439,10 @@
         sql (format "SELECT %s FROM (
                       SELECT fs.certname,
                              fp.name as name,
-                             fv.value,
+                             f.value,
                              env.environment
                       FROM factsets fs
                         INNER JOIN facts as f on fs.id = f.factset_id
-                        INNER JOIN fact_values as fv on f.fact_value_id = fv.id
                         INNER JOIN fact_paths as fp on f.fact_path_id = fp.id
                         LEFT OUTER JOIN environments as env on fs.environment_id = env.id
                       WHERE depth = 0) AS facts
@@ -481,6 +480,7 @@
          {:where  "catalog_resources.environment = ?"
           :params [value]}
 
+         ;; TODO handle node_state here?
          ;; {in,}active nodes.
          [["node" "active"]]
          {:where (format "catalogs.certname IN (%s)" (certname-names-query value))}
@@ -556,6 +556,7 @@
            {:where "facts.environment = ?"
             :params [value]}
 
+           ;; TODO handle node_state here?
            [["node" "active"]]
            {:where (format "facts.certname IN (%s)" (certname-names-query value))}
 
@@ -682,7 +683,7 @@
              ;; these fields allow NULL, which causes a change in semantics when
              ;; wrapped in a NOT(...) clause, so we have to be very explicit
              ;; about the NULL case.
-             [(field :guard #{"property" "message" "file" "line" "containing_class"})]
+             [(field :guard #{"property" "message" "file" "line" "containing_class" "corrective_change"})]
              (if-not (nil? value)
                {:where (format "resource_events.%s = ? AND resource_events.%s IS NOT NULL" field field)
                 :params [value] }
@@ -739,7 +740,7 @@
 (defn compile-event-count-equality
   "Compile an = predicate for event-count query.  The `path` represents
   the field to query against, and `value` is the value of the field."
-  [& [path value :as args]]
+  [fields & [path value :as args]]
   {:post [(map? %)
           (string? (:where %))]}
   (when-not (= (count args) 2)
@@ -747,7 +748,7 @@
             (i18n/tru "= requires exactly two arguments, but {0} were supplied" (count args)))))
   (let [db-field (utils/dashes->underscores path)]
     (match [db-field]
-           [(field :guard #{"successes" "failures" "noops" "skips"})]
+           [(field :guard fields)]
            {:where (format "%s = ?" field)
             :params [value]}
 
@@ -757,7 +758,7 @@
 (defn compile-event-count-inequality
   "Compile an inequality for an event-counts query (> < >= <=).  The `path`
   represents the field to query against, and the `value` is the value of the field."
-  [& [op path value :as args]]
+  [fields & [op path value :as args]]
   {:post [(map? %)
           (string? (:where %))]}
   (when-not (= (count args) 3)
@@ -765,7 +766,7 @@
             (i18n/tru "{0} requires exactly two arguments, but {1} were supplied"
                       op (dec (count args))))))
   (match [path]
-         [(field :guard #{"successes" "failures" "noops" "skips"})]
+         [(field :guard fields)]
          {:where (format "%s %s ?" field op)
           :params [value]}
 
@@ -835,8 +836,8 @@
 (defn event-count-ops
   "Maps resource event count operators to the functions implementing them.
   Returns nil if the operator is unknown."
-  [op]
+  [fields op]
   (let [op (str/lower-case op)]
     (cond
-     (= "=" op) compile-event-count-equality
-     (#{">" "<" ">=" "<="} op) (partial compile-event-count-inequality op))))
+     (= "=" op) (partial compile-event-count-equality fields)
+     (#{">" "<" ">=" "<="} op) (partial compile-event-count-inequality fields op))))
