@@ -1,6 +1,9 @@
 (ns puppetlabs.puppetdb.scf.hash
   (:require [puppetlabs.puppetdb.cheshire :as json]
-            [puppetlabs.kitchensink.core :as kitchensink]))
+            [puppetlabs.kitchensink.core :as kitchensink]
+            [puppetlabs.puppetdb.facts :refer [package-tuple hashed-package-tuple
+                                               package-tuple-hash]]
+            [schema.core :as s]))
 
 (defn generic-identity-string
   "Serialize a data structure into a format that can be hashed for uniqueness
@@ -8,8 +11,11 @@
   instead."
   [data]
   {:post [(string? %)]}
-  (-> (kitchensink/sort-nested-maps data)
-      (json/generate-string)))
+  (if (string? data)
+    data
+    (-> data
+        kitchensink/sort-nested-maps
+        json/generate-string)))
 
 (defn generic-identity-hash
   "Convert a data structure into a serialized format then grab a sha1 hash for
@@ -46,10 +52,6 @@
   {:post [(string? %)]}
   (generic-identity-hash [type title parameters]))
 
-;; Size of the cache is based on the number of unique resources in a
-;; "medium" site persona
-(def memoized-resource-identity-hash* (kitchensink/bounded-memoize resource-identity-hash* 40000))
-
 (defn resource-identity-hash
   "Compute a hash for a given resource that will uniquely identify it
   _for storage deduplication only_.
@@ -60,7 +62,7 @@
   [{:keys [type title parameters] :as resource}]
   {:pre  [(map? resource)]
    :post [(string? %)]}
-  (memoized-resource-identity-hash* type title parameters))
+  (resource-identity-hash* type title parameters))
 
 (defn catalog-resource-identity-format
   "Narrow `resource` to only contain the needed key/values for computing the hash of
@@ -91,6 +93,23 @@
           target-result))
       source-result)))
 
+(s/defn package-similarity-hash :- s/Str
+  "Creates a stable ordering of `packages` and computes a hash over
+  that structure"
+  [packages :- [hashed-package-tuple]]
+  (->> packages
+       (map package-tuple-hash)
+       sort
+       json/generate-string
+       kitchensink/utf8-string->sha1))
+
+(s/defn package-identity-hash :- hashed-package-tuple
+  [package :- package-tuple]
+  (conj package
+        (-> package
+            json/generate-string
+            kitchensink/utf8-string->sha1)))
+
 (defn catalog-similarity-format
   "Creates catalog map for the given `certname`, `resources` and `edges` with a
    stable ordering that can be used to create a hash consistently."
@@ -148,8 +167,7 @@
 
   This hash is useful for situations where you'd like to determine
   whether or not two reports contain the same things (certname,
-  configuration version, timestamps, events).
-  "
+  configuration version, timestamps, events)."
   [{:keys [certname puppet_version report_format configuration_version
            start_time end_time producer_timestamp resource_events transaction_uuid] :as report}]
   (generic-identity-hash
