@@ -51,21 +51,15 @@
             [clojure.tools.logging :as log]
             [clojure.string :as string]
             [puppetlabs.puppetdb.cheshire :as json]
+            [puppetlabs.puppetdb.utils :refer [flush-and-exit]]
             [puppetlabs.kitchensink.core :as kitchensink]
             [puppetlabs.puppetdb.scf.storage-utils :as sutils]
             [clojure.set :refer :all]
             [puppetlabs.puppetdb.time :refer [to-timestamp]]
             [clj-time.core :refer [now]]
             [puppetlabs.puppetdb.jdbc :as jdbc :refer [query-to-vec]]
-            [puppetlabs.puppetdb.config :as conf]))
-
-;; taken from storage.clj; preserved here in case of change
-(defn insert-records*
-  "Nil/empty safe insert-records, see java.jdbc's insert-records for more "
-  [table
-   record-coll]
-  (when (seq record-coll)
-    (apply jdbc/insert! table record-coll)))
+            [puppetlabs.puppetdb.config :as conf]
+            [puppetlabs.i18n.core :refer [trs]]))
 
 (defn init-through-2-3-8
   []
@@ -472,12 +466,12 @@
 
       (sql/create-table-ddl
        :factsets_transform
-       ["id" "bigint NOT NULL DEFAULT nextval('factsets_id_seq')"]
-       ["certname" "text NOT NULL"]
-       ["timestamp" "timestamp with time zone NOT NULL"]
-       ["environment_id" "bigint"]
-       ["hash" hash-type]
-       ["producer_timestamp" "timestamp with time zone NOT NULL"])
+       [["id" "bigint NOT NULL DEFAULT nextval('factsets_id_seq')"]
+        ["certname" "text NOT NULL"]
+        ["timestamp" "timestamp with time zone NOT NULL"]
+        ["environment_id" "bigint"]
+        ["hash" hash-type]
+        ["producer_timestamp" "timestamp with time zone NOT NULL"]])
 
       "INSERT INTO factsets_transform
          (id, certname, timestamp, environment_id, producer_timestamp)
@@ -486,14 +480,14 @@
 
       (sql/create-table-ddl
        :fact_values_transform
-       ["id" "bigint NOT NULL DEFAULT nextval('fact_values_id_seq')"]
-       ["value_hash" hash-type "NOT NULL"]
-       ["value_type_id" "bigint NOT NULL"]
-       ["value_integer" "bigint"]
-       ["value_float" "double precision"]
-       ["value_string" "text"]
-       ["value_boolean" "boolean"]
-       ["value_json" "text"])
+       [["id" "bigint NOT NULL DEFAULT nextval('fact_values_id_seq')"]
+        ["value_hash" hash-type "NOT NULL"]
+        ["value_type_id" "bigint NOT NULL"]
+        ["value_integer" "bigint"]
+        ["value_float" "double precision"]
+        ["value_string" "text"]
+        ["value_boolean" "boolean"]
+        ["value_json" "text"]])
 
       (str "INSERT INTO fact_values_transform
               (id, value_hash, value_type_id, value_integer, value_float,
@@ -504,8 +498,8 @@
                 FROM fact_values")
 
       (sql/create-table-ddl :resource_params_cache_transform
-                            ["resource" hash-type "NOT NULL"]
-                            ["parameters" "TEXT"])
+                            [["resource" hash-type "NOT NULL"]
+                             ["parameters" "TEXT"]])
 
       (str "INSERT INTO resource_params_cache_transform
               (resource, parameters)
@@ -514,14 +508,14 @@
 
       (sql/create-table-ddl
        :catalog_resources_transform
-       ["catalog_id" "bigint NOT NULL"]
-       ["resource" hash-type "NOT NULL"]
-       ["tags" (sutils/sql-array-type-string "TEXT") "NOT NULL"]
-       ["type" "TEXT" "NOT NULL"]
-       ["title" "TEXT" "NOT NULL"]
-       ["exported" "BOOLEAN" "NOT NULL"]
-       ["file" "TEXT"]
-       ["line" "INT"])
+       [["catalog_id" "bigint NOT NULL"]
+        ["resource" hash-type "NOT NULL"]
+        ["tags" (sutils/sql-array-type-string "TEXT") "NOT NULL"]
+        ["type" "TEXT" "NOT NULL"]
+        ["title" "TEXT" "NOT NULL"]
+        ["exported" "BOOLEAN" "NOT NULL"]
+        ["file" "TEXT"]
+        ["line" "INT"]])
 
       (str "INSERT INTO catalog_resources_transform
               (resource, catalog_id, tags, type, title, exported, file, line)
@@ -530,9 +524,9 @@
                 FROM catalog_resources")
 
       (sql/create-table-ddl :resource_params_transform
-                            ["resource" hash-type "NOT NULL"]
-                            ["name"  "TEXT" "NOT NULL"]
-                            ["value" "TEXT" "NOT NULL"])
+                            [["resource" hash-type "NOT NULL"]
+                             ["name"  "TEXT" "NOT NULL"]
+                             ["value" "TEXT" "NOT NULL"]])
 
       (str "INSERT INTO resource_params_transform
               (resource, name, value)
@@ -540,10 +534,10 @@
                 FROM resource_params")
 
       (sql/create-table-ddl :edges_transform
-                            ["certname" "TEXT" "NOT NULL"]
-                            ["source" hash-type "NOT NULL"]
-                            ["target" hash-type "NOT NULL"]
-                            ["type" "TEXT" "NOT NULL"])
+                            [["certname" "TEXT" "NOT NULL"]
+                             ["source" hash-type "NOT NULL"]
+                             ["target" hash-type "NOT NULL"]
+                             ["type" "TEXT" "NOT NULL"]])
 
       (str "INSERT INTO edges_transform (certname, source, target, type)
               SELECT certname, " (munge-hash "source") ",
@@ -554,15 +548,15 @@
 
       (sql/create-table-ddl
        :catalogs_transform
-       ["id" "bigint NOT NULL DEFAULT nextval('catalogs_id_seq')"]
-       ["hash" hash-type "NOT NULL"]
-       ["transaction_uuid" uuid-type]
-       ["certname" "text NOT NULL"]
-       ["producer_timestamp" "timestamp with time zone NOT NULL"]
-       ["api_version" "INTEGER NOT NULL"]
-       ["timestamp" "TIMESTAMP WITH TIME ZONE"]
-       ["catalog_version" "TEXT NOT NULL"]
-       ["environment_id" "bigint"])
+       [["id" "bigint NOT NULL DEFAULT nextval('catalogs_id_seq')"]
+        ["hash" hash-type "NOT NULL"]
+        ["transaction_uuid" uuid-type]
+        ["certname" "text NOT NULL"]
+        ["producer_timestamp" "timestamp with time zone NOT NULL"]
+        ["api_version" "INTEGER NOT NULL"]
+        ["timestamp" "TIMESTAMP WITH TIME ZONE"]
+        ["catalog_version" "TEXT NOT NULL"]
+        ["environment_id" "bigint"]])
 
       (str "INSERT INTO catalogs_transform
               (id, hash, transaction_uuid, certname, producer_timestamp,
@@ -578,24 +572,24 @@
 
       (sql/create-table-ddl
        :reports_transform
-       ["id" "bigint NOT NULL DEFAULT nextval('reports_id_seq')"]
-       ["hash" hash-type "NOT NULL"]
-       ["transaction_uuid" uuid-type]
-       ["certname" "text NOT NULL"]
-       ["puppet_version" "varchar(255) NOT NULL"]
-       ["report_format" "smallint NOT NULL"]
-       ["configuration_version" "varchar(255) NOT NULL"]
-       ["start_time" "timestamp with time zone NOT NULL"]
-       ["end_time" "timestamp with time zone NOT NULL"]
-       ["receive_time" "timestamp with time zone NOT NULL"]
-       ;; Insert a column in reports to be populated by boolean noop flag
-       ["noop" "boolean"]
-       ["environment_id" "bigint"]
-       ["status_id" "bigint"]
-       ;; Insert columns in reports to be populated by metrics and logs.
-       ;; Text for hsql, JSON for postgres.
-       ["metrics" json-type]
-       ["logs" json-type])
+       [["id" "bigint NOT NULL DEFAULT nextval('reports_id_seq')"]
+        ["hash" hash-type "NOT NULL"]
+        ["transaction_uuid" uuid-type]
+        ["certname" "text NOT NULL"]
+        ["puppet_version" "varchar(255) NOT NULL"]
+        ["report_format" "smallint NOT NULL"]
+        ["configuration_version" "varchar(255) NOT NULL"]
+        ["start_time" "timestamp with time zone NOT NULL"]
+        ["end_time" "timestamp with time zone NOT NULL"]
+        ["receive_time" "timestamp with time zone NOT NULL"]
+        ;; Insert a column in reports to be populated by boolean noop flag
+        ["noop" "boolean"]
+        ["environment_id" "bigint"]
+        ["status_id" "bigint"]
+        ;; Insert columns in reports to be populated by metrics and logs.
+        ;; Text for hsql, JSON for postgres.
+        ["metrics" json-type]
+        ["logs" json-type]])
 
       (str "INSERT INTO reports_transform (
             hash, certname, puppet_version, report_format, configuration_version,
@@ -608,19 +602,19 @@
 
       (sql/create-table-ddl
        :resource_events_transform
-       ["report_id" "bigint NOT NULL"]
-       ["status" "varchar(40) NOT NULL"]
-       ["timestamp" "timestamp with time zone NOT NULL"]
-       ["resource_type" "text NOT NULL"]
-       ["resource_title" "text NOT NULL"]
-       ["property" "varchar (40)"]
-       ["new_value" "text"]
-       ["old_value" "text"]
-       ["message" "text"]
-       ["file" "varchar(1024) DEFAULT NULL"]
-       ["line" "integer"]
-       ["containment_path" (sutils/sql-array-type-string "TEXT")]
-       ["containing_class" "varchar(255)"])
+       [["report_id" "bigint NOT NULL"]
+        ["status" "varchar(40) NOT NULL"]
+        ["timestamp" "timestamp with time zone NOT NULL"]
+        ["resource_type" "text NOT NULL"]
+        ["resource_title" "text NOT NULL"]
+        ["property" "varchar (40)"]
+        ["new_value" "text"]
+        ["old_value" "text"]
+        ["message" "text"]
+        ["file" "varchar(1024) DEFAULT NULL"]
+        ["line" "integer"]
+        ["containment_path" (sutils/sql-array-type-string "TEXT")]
+        ["containing_class" "varchar(255)"]])
 
       (str "INSERT INTO resource_events_transform (
             report_id, status, timestamp, resource_type, resource_title, property,
@@ -635,9 +629,9 @@
       (sql/create-table-ddl
        :certnames_transform
        ;; Rename the 'name' column of certnames to 'certname'.
-       ["certname" "text NOT NULL"]
-       ["latest_report_id" "bigint"]
-       ["deactivated" "timestamp with time zone"])
+       [["certname" "text NOT NULL"]
+        ["latest_report_id" "bigint"]
+        ["deactivated" "timestamp with time zone"]])
 
       (str "INSERT INTO certnames_transform(certname,latest_report_id,deactivated)
             SELECT c.name, rt.id as latest_report_id, c.deactivated FROM
@@ -802,12 +796,13 @@
         value-keys [:value_string :value_integer
                     :value_json :value_boolean
                     :value_float]]
-    (jdbc/with-query-results-cursor query
-      (fn [rs]
-        (->> rs
-             (map (partial coalesce-values value-keys))
-             (map update-value-json)
-             dorun)))
+    (jdbc/call-with-array-converted-query-rows
+     query
+     (fn [rows]
+       (->> rows
+            (map (partial coalesce-values value-keys))
+            (map update-value-json)
+            dorun)))
     (jdbc/do-commands
       "ALTER TABLE fact_values RENAME COLUMN value_json TO value")))
 
@@ -946,11 +941,11 @@
    (sql/create-table-ddl
     :certnames_transform
     ;; Rename the 'name' column of certnames to 'certname'.
-    ["id" "bigint NOT NULL PRIMARY KEY default nextval('certname_id_seq')"]
-    ["certname" "text NOT NULL UNIQUE"]
-    ["latest_report_id" "bigint"]
-    ["deactivated" "timestamp with time zone"]
-    ["expired" "timestamp with time zone"])
+    [["id" "bigint NOT NULL PRIMARY KEY default nextval('certname_id_seq')"]
+     ["certname" "text NOT NULL UNIQUE"]
+     ["latest_report_id" "bigint"]
+     ["deactivated" "timestamp with time zone"]
+     ["expired" "timestamp with time zone"]])
 
    "INSERT INTO certnames_transform
      (certname, latest_report_id, deactivated, expired)
@@ -978,20 +973,20 @@
   (jdbc/do-commands
    (sql/create-table-ddl
     :resource_events_transform
-    ["report_id" "bigint NOT NULL"]
-    ["certname_id" "bigint NOT NULL"]
-    ["status" "varchar(40) NOT NULL"]
-    ["timestamp" "timestamp with time zone NOT NULL"]
-    ["resource_type" "text NOT NULL"]
-    ["resource_title" "text NOT NULL"]
-    ["property" "varchar (40)"]
-    ["new_value" "text"]
-    ["old_value" "text"]
-    ["message" "text"]
-    ["file" "varchar(1024) DEFAULT NULL"]
-    ["line" "integer"]
-    ["containment_path" (sutils/sql-array-type-string "TEXT")]
-    ["containing_class" "varchar(255)"])
+    [["report_id" "bigint NOT NULL"]
+     ["certname_id" "bigint NOT NULL"]
+     ["status" "varchar(40) NOT NULL"]
+     ["timestamp" "timestamp with time zone NOT NULL"]
+     ["resource_type" "text NOT NULL"]
+     ["resource_title" "text NOT NULL"]
+     ["property" "varchar (40)"]
+     ["new_value" "text"]
+     ["old_value" "text"]
+     ["message" "text"]
+     ["file" "varchar(1024) DEFAULT NULL"]
+     ["line" "integer"]
+     ["containment_path" (sutils/sql-array-type-string "TEXT")]
+     ["containing_class" "varchar(255)"]])
 
    "INSERT INTO resource_events_transform (
        report_id, certname_id, status, timestamp, resource_type, resource_title,
@@ -1037,13 +1032,19 @@
   (jdbc/do-commands
     "CREATE INDEX idx_certnames_latest_report_id on certnames(latest_report_id)"))
 
+(defn index-certnames-unique-latest-report-id
+  []
+  (jdbc/do-commands
+    "DROP INDEX IF EXISTS idx_certnames_latest_report_id"
+    "CREATE UNIQUE INDEX idx_certnames_latest_report_id on certnames(latest_report_id)"))
+
 (defn add-producer-to-reports-catalogs-and-factsets
   []
   (jdbc/do-commands
     (sql/create-table-ddl
       :producers
-      ["id" "bigint PRIMARY KEY"]
-      ["name" "text NOT NULL UNIQUE"])
+      [["id" "bigint PRIMARY KEY"]
+       ["name" "text NOT NULL UNIQUE"]])
     "CREATE SEQUENCE producers_id_seq CYCLE"
     "ALTER TABLE producers ALTER COLUMN id SET DEFAULT nextval('producers_id_seq')"
     "ALTER TABLE reports ADD COLUMN producer_id bigint"
@@ -1087,19 +1088,18 @@
 (defn migrate-through-app
   [table1 table2 column-list munge-fn]
   (let [columns (string/join "," column-list)]
-    (jdbc/with-query-results-cursor
-      [(format "select %s from %s" columns (name table1))]
-      #(->> %
-            (map munge-fn)
-            (map (partial jdbc/insert! (name table2)))
-            dorun))))
+    (jdbc/call-with-array-converted-query-rows
+     [(format "select %s from %s" columns (name table1))]
+     #(->> %
+           (map munge-fn)
+           (jdbc/insert-multi! (name table2))))))
 
 (defn resource-params-cache-parameters-to-jsonb
   []
   (jdbc/do-commands
     (sql/create-table-ddl :resource_params_cache_transform
-                          ["resource" "bytea NOT NULL"]
-                          ["parameters" "jsonb"]))
+                          [["resource" "bytea NOT NULL"]
+                           ["parameters" "jsonb"]]))
 
   (migrate-through-app
     :resource_params_cache
@@ -1128,14 +1128,14 @@
   []
   (jdbc/do-commands
     (sql/create-table-ddl :fact_values_transform
-                          ["id" "bigint NOT NULL PRIMARY KEY DEFAULT nextval('fact_values_id_seq')"]
-                          ["value_hash" "bytea NOT NULL UNIQUE"]
-                          ["value_type_id" "bigint NOT NULL"]
-                          ["value_integer" "bigint"]
-                          ["value_float" "double precision"]
-                          ["value_string" "text"]
-                          ["value_boolean" "boolean"]
-                          ["value" "jsonb"]))
+                          [["id" "bigint NOT NULL PRIMARY KEY DEFAULT nextval('fact_values_id_seq')"]
+                           ["value_hash" "bytea NOT NULL UNIQUE"]
+                           ["value_type_id" "bigint NOT NULL"]
+                           ["value_integer" "bigint"]
+                           ["value_float" "double precision"]
+                           ["value_string" "text"]
+                           ["value_boolean" "boolean"]
+                           ["value" "jsonb"]]))
 
   (migrate-through-app
     :fact_values
@@ -1168,6 +1168,139 @@
   (jdbc/do-commands
     "CREATE INDEX resource_events_status_for_corrective_change_idx ON resource_events (status) WHERE corrective_change"))
 
+(defn drop-resource-events-resource-type-idx
+  []
+  (jdbc/do-commands
+    "DROP INDEX IF EXISTS resource_events_resource_type_idx"))
+
+(defn merge-fact-values-into-facts
+  []
+  (jdbc/do-commands
+   ["create table facts_transform"
+    "  (factset_id bigint not null,"
+    "   fact_path_id bigint not null,"
+    "   value_integer bigint,"
+    "   value_float double precision,"
+    "   value_type_id bigint not null,"
+    "   value_string text,"
+    "   value jsonb,"
+    "   large_value_hash bytea,"
+    "   value_boolean boolean)"]
+
+   ["insert into facts_transform"
+    "  (factset_id,"
+    "   fact_path_id,"
+    "   value_integer,"
+    "   value_float,"
+    "   value_type_id,"
+    "   value_string,"
+    "   value,"
+    "   large_value_hash,"
+    "   value_boolean)"
+    "  select factset_id, fact_path_id,"
+    "         value_integer, value_float, value_type_id, value_string, value,"
+    "         case when pg_column_size(value) >= 50"
+    "              then value_hash"
+    "         end,"
+    "         value_boolean"
+    "    from facts inner join fact_values on fact_value_id = fact_values.id"]
+
+   "drop index facts_fact_value_id_idx"
+   "drop table facts"
+   "drop table fact_values"
+   "drop sequence fact_values_id_seq"
+
+    "alter table facts_transform rename to facts"
+
+    "create index facts_factset_id_idx on facts using btree (factset_id)"
+    "create index facts_fact_path_id_idx on facts using btree (fact_path_id)"
+    "create index facts_value_integer_idx on facts(value_integer)"
+    "create index facts_value_float_idx on facts(value_float)"
+
+    ["alter table facts add constraint facts_value_type_id_fk"
+     "  foreign key (value_type_id) references value_types (id) match simple"
+     "    on update restrict on delete restrict"]))
+
+(defn add-package-tables []
+  (jdbc/do-commands
+   ["create table packages "
+    "  (id bigint PRIMARY KEY, "
+    "   hash bytea, "
+    "   name text not null, "
+    "   provider text not null, "
+    "   version text not null)"]
+
+   "ALTER TABLE certnames ADD COLUMN package_hash bytea"
+
+   "CREATE SEQUENCE package_id_seq CYCLE"
+   "ALTER TABLE packages ALTER COLUMN id SET DEFAULT nextval('package_id_seq')"
+
+   ["ALTER TABLE ONLY packages "
+    "ADD CONSTRAINT package_hash_key UNIQUE (hash)"]
+
+   ["create table certname_packages"
+    "  (certname_id bigint not null,"
+    "   package_id bigint not null,"
+    "   PRIMARY KEY (certname_id, package_id),"
+    "   FOREIGN KEY (certname_id) REFERENCES certnames(id),"
+    "   FOREIGN KEY (package_id) REFERENCES packages(id))"]
+
+   "create index certname_package_reverse_idx on certname_packages using btree (package_id, certname_id)"
+   "create index packages_name_idx on packages using btree (name)"))
+
+(defn add-gin-index-on-resource-params-cache []
+  (jdbc/do-commands
+    "create index resource_params_cache_parameters_idx on resource_params_cache using gin (parameters)"))
+
+(defn improve-facts-factset-id-index []
+  (jdbc/do-commands
+    "DROP INDEX IF EXISTS facts_factset_id_idx"
+    "DROP INDEX IF EXISTS facts_factset_id_fact_path_id_idx"
+    "CREATE INDEX facts_factset_id_fact_path_id_idx ON facts(factset_id, fact_path_id)"))
+
+(defn fix-missing-edges-fk-constraint []
+  (when-not (sutils/constraint-exists? "certnames" "edges_certname_fkey")
+    (log/info (trs "Cleaning up orphaned edges"))
+
+    (jdbc/do-commands
+     (str "SELECT e.*"
+          "  INTO edges_transform"
+          "  FROM edges e"
+          "  INNER JOIN certnames c ON e.certname = c.certname")
+     (str "ALTER TABLE edges_transform"
+          "  ALTER COLUMN certname SET NOT NULL,"
+          "  ALTER COLUMN source SET NOT NULL,"
+          "  ALTER COLUMN target SET NOT NULL,"
+          "  ALTER COLUMN type SET NOT NULL")
+     (str "DROP TABLE edges")
+     (str "ALTER TABLE edges_transform RENAME TO edges")
+     (str "ALTER TABLE ONLY edges ADD CONSTRAINT edges_certname_fkey"
+          "  FOREIGN KEY (certname)"
+          "  REFERENCES certnames(certname)"
+          "  ON DELETE CASCADE")
+     (str "ALTER TABLE ONLY edges"
+          "  ADD CONSTRAINT edges_certname_source_target_type_unique_key"
+          "  UNIQUE (certname, source, target, type)"))))
+(defn add-latest-report-timestamp-to-certnames []
+  (jdbc/do-commands
+    "DROP INDEX IF EXISTS idx_certnames_latest_report_timestamp"
+    "ALTER TABLE certnames DROP COLUMN IF EXISTS latest_report_timestamp"
+    "ALTER TABLE certnames ADD COLUMN latest_report_timestamp timestamp with time zone"
+    "CREATE INDEX idx_certnames_latest_report_timestamp ON certnames(latest_report_timestamp)"))
+
+(defn reports-partial-indices
+  []
+  (jdbc/do-commands
+   "create index reports_noop_idx on reports(noop) where noop = true"
+   "create index reports_cached_catalog_status_on_fail on reports(cached_catalog_status) where cached_catalog_status = 'on_failure'"))
+
+(defn add-job-id []
+  (jdbc/do-commands
+   "alter table reports add column job_id text default null"
+   "alter table catalogs add column job_id text default null"
+   "create index reports_job_id_idx on reports(job_id) where job_id is not null"
+   "create index catalogs_job_id_idx on catalogs(job_id) where job_id is not null"))
+
 (def migrations
   "The available migrations, as a map from migration version to migration function."
   {28 init-through-2-3-8
@@ -1198,8 +1331,17 @@
    50 remove-historical-catalogs
    51 fact-values-value-to-jsonb
    52 resource-params-cache-parameters-to-jsonb
-   53 add-corrective-change-index})
-
+   53 add-corrective-change-index
+   54 drop-resource-events-resource-type-idx
+   55 index-certnames-unique-latest-report-id
+   56 merge-fact-values-into-facts
+   57 add-package-tables
+   58 add-gin-index-on-resource-params-cache
+   59 improve-facts-factset-id-index
+   60 fix-missing-edges-fk-constraint
+   61 add-latest-report-timestamp-to-certnames
+   62 reports-partial-indices
+   63 add-job-id})
 
 (def desired-schema-version (apply max (keys migrations)))
 
@@ -1245,20 +1387,20 @@
 
 (defn- sql-or-die
   "Calls (f) and returns its result.  If f throws an SQLException,
-  logs the exception and its parent and then calls (System/exit 1)"
+  logs the exception and its parent and then calls (flush-and-exit 1)"
   [f]
   ;; Here we've preserved existing behavior, but this may warrant
   ;; further consideration later.  If possible, we might want to
-  ;; avoid System/exit in favor of careful exception
+  ;; avoid flush-and-exit in favor of careful exception
   ;; handling (cf. PDB-1118).
   (try
     (f)
     (catch java.sql.SQLException e
-      (log/error e "Caught SQLException during migration")
+      (log/error e (trs "Caught SQLException during migration"))
       (when-let [next (.getNextException e)]
-        (log/error next "Unravelled exception"))
+        (log/error next (trs "Unravelled exception")))
       (binding [*out* *err*] (flush)) (flush)
-      (System/exit 1))))
+      (flush-and-exit 1))))
 
 (defn previous-migrations
   "Returns the list of migration numbers that existed before the
@@ -1292,26 +1434,28 @@
     (when (and latest-applied-migration
                (< latest-applied-migration (first known-migrations)))
       (throw (IllegalStateException.
-              (format (str "Found an old and unuspported database migration (migration number %s)."
-                           " PuppetDB only supports upgrading from the previous major version to the current major version."
-                           " As an example, users wanting to upgrade from 2.x to 4.x should first upgrade to 3.x.")
-                      latest-applied-migration))))
+              (str
+               (trs "Found an old and unuspported database migration (migration number {0})." latest-applied-migration)
+               " "
+               (trs "PuppetDB only supports upgrading from the previous major version to the current major version.")
+               " "
+               (trs "As an example, users wanting to upgrade from 2.x to 4.x should first upgrade to 3.x.")))))
 
     (when-let [unexpected (first (unrecognized-migrations applied-migration-versions known-migrations))]
       (throw (IllegalStateException.
-              (format "Your PuppetDB database contains a schema migration numbered %d, but this version of PuppetDB does not recognize that version."
-                      unexpected))))
+              (trs "Your PuppetDB database contains a schema migration numbered {0}, but this version of PuppetDB does not recognize that version."
+                   unexpected))))
 
     (if-let [pending (seq (pending-migrations))]
       (do
         (jdbc/with-db-transaction []
           (doseq [[version migration] pending]
-            (log/infof "Applying database migration version %d" version)
+            (log/info (trs "Applying database migration version {0}" version))
             (sql-or-die (fn [] (migration) (record-migration! version)))))
         (sutils/analyze-small-tables small-tables)
         true)
       (do
-        (log/info "There are no pending migrations")
+        (log/info (trs "There are no pending migrations"))
         false))))
 
 ;; SPECIAL INDEX HANDLING
@@ -1320,13 +1464,22 @@
   "Create trgm indexes if they do not currently exist."
   []
   (when-not (sutils/index-exists? "fact_paths_path_trgm")
-    (log/info "Creating additional index `fact_paths_path_trgm`")
+    (log/info (trs "Creating additional index `fact_paths_path_trgm`"))
     (jdbc/do-commands
      "CREATE INDEX fact_paths_path_trgm ON fact_paths USING gist (path gist_trgm_ops)"))
-  (when-not (sutils/index-exists? "fact_values_string_trgm")
-    (log/info "Creating additional index `fact_values_string_trgm`")
+  (jdbc/do-commands
+   "drop index if exists fact_values_string_trgm")
+  (when-not (sutils/index-exists? "facts_value_string_trgm")
+    (log/info (trs "Creating additional index `facts_value_string_trgm`"))
     (jdbc/do-commands
-     "CREATE INDEX fact_values_string_trgm ON fact_values USING gin (value_string gin_trgm_ops)")))
+     ["create index facts_value_string_trgm on facts"
+      "  using gin (value_string gin_trgm_ops)"]))
+
+  (when-not (sutils/index-exists? "packages_name_trgm")
+    (log/info (trs "Creating additional index `packages_name_trgm`"))
+    (jdbc/do-commands
+     ["create index packages_name_trgm on packages"
+      "  using gin (name gin_trgm_ops)"])))
 
 (defn indexes!
   "Create missing indexes for applicable database platforms."
@@ -1336,9 +1489,8 @@
       (trgm-indexes!)
       (log/warn
        (str
-        "Missing PostgreSQL extension `pg_trgm`\n\n"
-        "We are unable to create the recommended pg_trgm indexes due to\n"
-        "the extension not being installed correctly. Run the command:\n\n"
-        "    CREATE EXTENSION pg_trgm;\n\n"
-        "as the database super user on the PuppetDB database to correct\n"
-        "this, then restart PuppetDB.\n")))))
+        (trs "Missing PostgreSQL extension `pg_trgm`")
+        "\n\n"
+        (trs "We are unable to create the recommended pg_trgm indexes due to\nthe extension not being installed correctly.")
+        " "
+        (trs " Run the command:\n\n    CREATE EXTENSION pg_trgm;\n\nas the database super user on the PuppetDB database to correct\nthis, then restart PuppetDB.\n"))))))
