@@ -3,22 +3,22 @@
                                      HttpConfiguration HttpConnectionFactory
                                      ConnectionFactory AbstractConnectionFactory)
            (org.eclipse.jetty.server.handler AbstractHandler ContextHandler HandlerCollection
-                                             ContextHandlerCollection AllowSymLinkAliasChecker StatisticsHandler)
+                                             ContextHandlerCollection AllowSymLinkAliasChecker
+                                             StatisticsHandler HandlerWrapper)
+           (org.eclipse.jetty.server.handler.gzip GzipHandler)
            (org.eclipse.jetty.util.resource Resource)
            (org.eclipse.jetty.util.thread QueuedThreadPool)
            (org.eclipse.jetty.util.ssl SslContextFactory)
            (javax.servlet.http HttpServletResponse)
            (java.util.concurrent TimeoutException)
-           (org.eclipse.jetty.servlets.gzip GzipHandler)
            (org.eclipse.jetty.servlet ServletContextHandler ServletHolder DefaultServlet)
            (org.eclipse.jetty.webapp WebAppContext)
-           (java.util HashSet)
            (org.eclipse.jetty.http MimeTypes HttpHeader HttpHeaderValue)
            (javax.servlet Servlet ServletContextListener)
            (org.eclipse.jetty.proxy ProxyServlet)
            (java.net URI)
            (java.security Security)
-           (org.eclipse.jetty.client HttpClient)
+           (org.eclipse.jetty.client HttpClient RedirectProtocolHandler)
            (clojure.lang Atom)
            (java.lang.management ManagementFactory)
            (org.eclipse.jetty.jmx MBeanContainer)
@@ -33,7 +33,8 @@
             [puppetlabs.trapperkeeper.services.webserver.experimental.jetty9-websockets :as websockets]
             [puppetlabs.trapperkeeper.services.webserver.normalized-uri-helpers
              :as normalized-uri-helpers]
-            [schema.core :as schema]))
+            [schema.core :as schema]
+            [puppetlabs.i18n.core :as i18n]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; JDK SecurityProvider Hack
@@ -48,11 +49,11 @@
     (let [klass     (Class/forName "sun.security.pkcs11.SunPKCS11")
           blacklist (filter #(instance? klass %) (Security/getProviders))]
       (doseq [provider blacklist]
-        (log/info (str "Removing buggy security provider " provider))
+        (log/info (i18n/trs "Removing buggy security provider {0}" provider))
         (Security/removeProvider (.getName provider))))
     (catch ClassNotFoundException e)
     (catch Throwable e
-      (log/error e "Could not remove security providers; HTTPS may not work!"))))
+      (log/error e (i18n/trs "Could not remove security providers; HTTPS may not work!")))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Schemas
@@ -173,7 +174,7 @@
                                      :overrides-read-by-webserver
                                      true))]
     (doseq [key (keys overrides)]
-      (log/info (str "webserver config overridden for key '" (name key) "'")))
+      (log/info (i18n/trs "webserver config overridden for key ''{0}''" (name key))))
     (merge options overrides)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -185,9 +186,7 @@
   [{:keys [keystore-config client-auth ssl-crl-path cipher-suites protocols]}
    :- config/WebserverSslContextFactory]
   (if (some #(= "sslv3" %) (map str/lower-case protocols))
-    (log/warn (str "`ssl-protocols` contains SSLv3, a protocol with known "
-                  "vulnerabilities; we recommend removing it from the "
-                  "`ssl-protocols` list")))
+    (log/warn (i18n/trs "`ssl-protocols` contains SSLv3, a protocol with known vulnerabilities; we recommend removing it from the `ssl-protocols` list")))
 
   (let [context (doto (SslContextFactory.)
                   (.setKeyStore (:keystore keystore-config))
@@ -316,7 +315,7 @@
         ;;
         ;; The QueuedThreadPool constructor sets the `queue-capacity` and
         ;; `queue-grow-by` based on the minimum number of threads available
-        ;; in the pool.  See https://github.com/eclipse/jetty.project/blob/jetty-9.2.10.v20150310/jetty-util/src/main/java/org/eclipse/jetty/util/thread/QueuedThreadPool.java#L92-L96.
+        ;; in the pool.  See https://github.com/eclipse/jetty.project/blob/jetty-9.4.1.v20170120/jetty-util/src/main/java/org/eclipse/jetty/util/thread/QueuedThreadPool.java#L101-L105.
         ;; That algorithm is essentially duplicated here, with the only
         ;; difference being that if `queue-max-size` is smaller than the
         ;; minimum number of threads, the `queue-capacity` and `queue-grow-by`
@@ -383,7 +382,7 @@
                  (.startsWith % "video/"))
             (MimeTypes/getKnownMimeTypes))
     (conj "application/compress" "application/zip" "application/gzip" "text/event-stream")
-    (HashSet.)))
+    (into-array)))
 
 (defn- gzip-handler
   "Given a handler, wrap it with a GzipHandler that will compress the response
@@ -391,8 +390,7 @@
   [handler]
   (doto (GzipHandler.)
     (.setHandler handler)
-    (.setMimeTypes (gzip-excluded-mime-types))
-    (.setExcludeMimeTypes true)))
+    (.setExcludedMimeTypes (gzip-excluded-mime-types))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Handler Helper Functions
@@ -438,7 +436,7 @@
                                     (:ssl-config options)))
         {:keys [request-buffer-size idle-timeout]} options]
     (proxy [ProxyServlet] []
-      (rewriteURI [req]
+      (rewriteTarget [req]
         (let [query (.getQueryString req)
               scheme (let [target-scheme (:scheme options)]
                        (condp = target-scheme
@@ -459,8 +457,8 @@
                                  (codec/url-decode (str query))
                                  nil)]
             (if-let [rewrite-uri-callback-fn (:rewrite-uri-callback-fn options)]
-              (rewrite-uri-callback-fn target-uri req)
-              target-uri))))
+              (str (rewrite-uri-callback-fn target-uri req))
+              (str target-uri)))))
 
       (newHttpClient []
         (let [client (if custom-ssl-ctxt-factory
@@ -478,33 +476,36 @@
               timeout (when idle-timeout
                         (* 1000 idle-timeout))]
           (if (:follow-redirects options)
-            (.setFollowRedirects client true)
+            (do
+              (.setFollowRedirects client true)
+              (.put (.getProtocolHandlers client) (RedirectProtocolHandler. client)))
             (.setFollowRedirects client false))
           (when timeout
             (.setIdleTimeout client timeout))
           client))
 
-      (customizeProxyRequest [proxy-req req]
+      (sendProxyRequest [req resp proxy-req]
         (if-let [callback-fn (:callback-fn options)]
-         (callback-fn proxy-req req)))
+         (callback-fn proxy-req req))
+       (proxy-super sendProxyRequest req resp proxy-req))
 
       ;; The implementation of onResponseFailure is duplicated heavily from:
-      ;; https://github.com/eclipse/jetty.project/blob/jetty-9.2.10.v20150310/jetty-proxy/src/main/java/org/eclipse/jetty/proxy/AbstractProxyServlet.java#L576-L607
+      ;; https://github.com/eclipse/jetty.project/blob/jetty-9.4.1.v20170120/jetty-proxy/src/main/java/org/eclipse/jetty/proxy/AbstractProxyServlet.java#L624-L658
       ;; The only significant difference is that a 'failure-callback-fn', if
       ;; defined in options, is invoked just prior to completing the async
       ;; context for cases that the response was not already committed upstream.
-      (onResponseFailure [req resp proxy-resp failure]
+      (onProxyResponseFailure [req resp proxy-resp failure]
         (do
           (let [request-id    (.getRequestId this req)
                 async-context (.getAsyncContext req)]
-            (log/debug failure (str request-id " proxying failed"))
+            (log/debug failure (i18n/trs "{0} proxying failed" request-id))
             (if (.isCommitted resp)
               (try
                 (.sendError resp -1)
                 (.complete async-context)
                 (catch IOException _
-                  (log/debug failure (str request-id
-                                          " could not close the connection"))))
+                  (log/debug failure (i18n/trs "{0} could not close the connection"
+                                               request-id))))
               (do
                 (.resetBuffer resp)
                 (if (instance? TimeoutException failure)
@@ -526,6 +527,28 @@
                                                    [endpoint-map]
                                                    (conj % endpoint-map))))
 
+(schema/defn ^:always-validate max-request-body-size-handler*
+  [handler :- Handler
+   max-size :- schema/Int]
+  (proxy [HandlerWrapper] []
+    (handle [target ^Request base-request request response]
+      (let [request-size (.getContentLength base-request)]
+        (if (> request-size max-size)
+          (do
+            (.setStatus response HttpServletResponse/SC_REQUEST_ENTITY_TOO_LARGE)
+            (.setHandled base-request true))
+          (.handle handler target base-request request response))))))
+
+(schema/defn ^:always-validate max-request-body-size-handler
+  "Wrap a max-request-body-size handler around the supplied handler.  The
+  handler returns a 413 (request entity too large) error if the Content-Length
+  HTTP header on the incoming request exceeds the value specified as the
+  max-size parameter."
+  [handler :- Handler
+   max-size :- schema/Int]
+  (doto (max-request-body-size-handler* handler max-size)
+    (.setHandler handler)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
 
@@ -557,25 +580,23 @@
                                      :overrides-read-by-webserver
                                      true))]
     (doseq [key (keys overrides)]
-      (log/info (str "webserver config overridden for key '" (name key) "'")))
+      (log/info (i18n/trs "webserver config overridden for key ''{0}''" (name key))))
     (merge options overrides)))
 
 (schema/defn ^:always-validate shutdown
   [{:keys [server] :as webserver-context} :- ServerContext]
   (when-let [mbean-container (:mbean-container @(:state webserver-context))]
-    (log/debug "Cleaning up JMX MBean container")
+    (log/debug (i18n/trs "Cleaning up JMX MBean container"))
     (.destroy mbean-container)
     (swap! (:state webserver-context) assoc :mbean-container nil))
   (when (started? webserver-context)
-    (log/info "Shutting down web server.")
+    (log/info (i18n/trs "Shutting down web server."))
     (try
       (.stop server)
       (catch TimeoutException e
-        (log/errorf e
-                    (str "Web server failed to shut down gracefully in configured "
-                         "timeout period (%s); cancelling remaining requests.")
-                    (.getStopTimeout server))))
-    (log/info "Web server shutdown")))
+        (log/error e (i18n/trs "Web server failed to shut down gracefully in configured timeout period ({0}); cancelling remaining requests."
+                               (.getStopTimeout server)))))
+    (log/info (i18n/trs "Web server shutdown"))))
 
 (schema/defn ^:always-validate
   create-webserver :- ServerContext
@@ -629,13 +650,18 @@
     (.setHandlers hc (into-array Handler [(:handlers webserver-context)]))
     (let [shutdown-timeout (when (not (nil? (:shutdown-timeout-seconds options)))
                              (* 1000 (:shutdown-timeout-seconds options)))
-          maybe-zipped (if (or (not (contains? options :gzip-enable))
-                               (:gzip-enable options))
+          maybe-zipped (if (:gzip-enable options true)
                          (gzip-handler hc)
                          hc)
+          maybe-size-restricted (if-let [max-size (:request-body-max-size
+                                                   options)]
+                                  (max-request-body-size-handler
+                                   maybe-zipped
+                                   max-size)
+                                  maybe-zipped)
           maybe-logged (if log-handler
-                         (doto log-handler (.setHandler hc))
-                         maybe-zipped)
+                         (doto log-handler (.setHandler maybe-size-restricted))
+                         maybe-size-restricted)
           statistics-handler (if (or (nil? shutdown-timeout) (pos? shutdown-timeout))
                                (doto (StatisticsHandler.)
                                  (.setHandler maybe-logged))
@@ -653,13 +679,11 @@
   [webserver-context :- ServerContext
    config :- config/WebserverRawConfig]
   (let [webserver-context (create-webserver webserver-context config)]
-    (log/info "Starting web server.")
+    (log/info (i18n/trs "Starting web server."))
     (try
       (.start (:server webserver-context))
       (catch Exception e
-        (log/error
-          e
-          "Encountered error starting web server, so shutting down")
+        (log/error e (i18n/trs "Encountered error starting web server, so shutting down"))
         (shutdown webserver-context)
         (throw e)))
     webserver-context))
@@ -680,8 +704,9 @@
          enable-trailing-slash-redirect? (:enable-trailing-slash-redirect? options)
          normalize-request-uri? (:normalize-request-uri? options)]
      (.setBaseResource handler (Resource/newResource base-path))
-     (when follow-links?
-       (.addAliasCheck handler (AllowSymLinkAliasChecker.)))
+     (if follow-links?
+       (.setAliasChecks handler [(AllowSymLinkAliasChecker.)])
+       (.clearAliasChecks handler))
      ;; register servlet context listeners (if any)
      (when-not (nil? context-listeners)
        (dorun (map #(.addEventListener handler %) context-listeners)))
@@ -878,20 +903,16 @@
                (if (nil? (:overrides %))
                  (throw
                    (IllegalStateException.
-                     (str "overrides cannot be set because webserver has "
-                          "already processed the config")))
+                     (i18n/trs "overrides cannot be set because webserver has already processed the config")))
                  (throw
                    (IllegalStateException.
-                     (str "overrides cannot be set because they have "
-                          "already been set and webserver has already "
-                          "processed the config"))))
+                     (i18n/trs "overrides cannot be set because they have already been set and webserver has already processed the config"))))
              (nil? (:overrides %))
                (assoc % :overrides overrides)
              :else
                (throw
                  (IllegalStateException.
-                   (str "overrides cannot be set because they have "
-                        "already been set")))))))
+                   (i18n/trs "overrides cannot be set because they have already been set")))))))
 
 (schema/defn ^:always-validate join
   [webserver-context :- ServerContext]
@@ -921,8 +942,7 @@
                     server-id)]
     (when-not server-id
       (throw (IllegalArgumentException.
-               (str "no server-id was specified for this operation and "
-                    "no default server was specified in the configuration"))))
+               (i18n/trs "no server-id was specified for this operation and no default server was specified in the configuration"))))
     (server-id (:jetty9-servers service-context))))
 
 (defn get-default-server-from-config
